/* bytepass.c - handles the connection to the remote */
/* server and passes the data between the hosts      */

#include <stdlib.h>
#include <stdio.h>
#include <netinet/in.h>
#include <signal.h>
#include <netdb.h>
#include <sys/select.h>
#include <sys/socket.h>
#include <sys/wait.h>
#include <string.h>
#include <time.h>

#define min(a,b) ((a) < (b) ? (a) : (b))
#define max(a,b) ((a) > (b) ? (a) : (b))

#define SA struct sockaddr	// Improves readability of code, eh

void signalpipe(void);          // Handles a broken pipe
void sig_chld(void);            // Handles SIGCHLD
void signalinterrupt(void);
void setupsignals(void);        // Sets up signal handlers
void syntax();
void setupsignals();
int bytepass(struct hostent *, char *, char *, short);

int bytepass(struct hostent *hptr, char *host, char *port, short hex_flag)
{
	extern int listeningfd, clientfd, datafd;	// Attach to globals
	struct hostent *rhptr;
	struct in_addr **pptr;		// Address list
	struct sockaddr_in raddr;	// Remote address
	char buffer[512];		// 512-Byte Buffers
	int maxfd, n;
	fd_set rset;
	time_t rawtime;
	char szBuffer[256];
	struct tm* loc = NULL;

	bzero(&raddr, sizeof(raddr));	// Clear address structure

	// Get the current time and format it
	time(&rawtime);
	loc = localtime(&rawtime);
	strftime(szBuffer, sizeof(szBuffer)-1, "%D:%H:%M:%S", loc);
	
	printf("%s: Connection from %s\n",
			szBuffer,
			hptr->h_name);

	if((rhptr = gethostbyname(host)) == NULL)
	{
		fprintf(stderr, "Error: Couldn't resolve %s, forward failed\n",
			host);
		return -1;
	}

	pptr = (struct in_addr **) rhptr->h_addr_list;
	memcpy(&raddr.sin_addr, *pptr, sizeof(struct in_addr));

	raddr.sin_port = htons(atoi(port));
	raddr.sin_family = AF_INET;

	if((datafd = socket(AF_INET, SOCK_STREAM, 0)) < 0)
	{
		fprintf(stderr, "Error: Couldn't create a new socket for client %s\n",
			hptr->h_name);
		return -1;
	}

	// Make connection to remote host
	if(connect(datafd, (SA *) &raddr, sizeof(raddr)) < 0)
	{
		fprintf(stderr, "Error: Connection failed to %s:%s\n",
			host, port);
		return -1;
	}

	// Transfer data between the two hosts
	FD_ZERO(&rset);
	while(1)
	{
		// Add the two socket descriptors to the array
		FD_SET(clientfd, &rset);
		FD_SET(datafd, &rset);

		// Computer the max descriptor number to be detected
		maxfd = max(clientfd, datafd) + 1;

		// Wait forever until one of the descriptors unblocks
		select(maxfd, &rset, NULL, NULL, NULL);

		// If the remote host comes back
		if(FD_ISSET(datafd, &rset))
		{
			if((n = read(datafd, &buffer, sizeof(buffer))) > 0)
			{
//				if(hex_flag)
//					hexwrite(clientfd, &buffer, n);

				if(write(clientfd, &buffer, n) < 1)
				{
					fprintf(stderr, "Error: Bad write to client %s\n",
						hptr->h_name);
					return -1;
				}
			}
			else
			{
				// Get the current time and format it
				time(&rawtime);
				loc = localtime(&rawtime);
				strftime(szBuffer, sizeof(szBuffer)-1, "%D:%H:%M:%S", loc);
		
				fprintf(stderr, "%s: Server (%s) initiated disconnect of client (%s)\n",
					szBuffer, rhptr->h_name, hptr->h_name);
				return 0;
			}
		}
				
		// If the client starts sending something
		if(FD_ISSET(clientfd, &rset))
		{
			if((n = read(clientfd, &buffer, sizeof(buffer))) > 0)
			{
//				if(hex_flag)
//					hexwrite(datafd, &buffer, n);
			

				if(write(datafd, &buffer, n) < 1)
				{
					fprintf(stderr, "Error: Bad write to server %s\n",
						rhptr->h_name);
					return -1;
				}
			}
			else
			{
				// Get the current time and format it
				time(&rawtime);
				loc = localtime(&rawtime);
				strftime(szBuffer, sizeof(szBuffer)-1, "%D:%H:%M:%S", loc);
		
				fprintf(stderr, "%s: Client (%s) initiated disconnect from server (%s)\n",
					szBuffer, hptr->h_name, rhptr->h_name);
				return 0;
			}
		}
	}
}	
